( cd usr/lib ; rm -rf libdb-4.2.so )
( cd usr/lib ; ln -sf /lib/libdb-4.2.so libdb-4.2.so )
( cd usr/lib ; rm -rf libdb-4.so )
( cd usr/lib ; ln -sf libdb-4.2.so libdb-4.so )
( cd usr/lib ; rm -rf libdb-4.a )
( cd usr/lib ; ln -sf libdb-4.2.a libdb-4.a )
( cd usr/lib ; rm -rf libdb4.a )
( cd usr/lib ; ln -sf libdb-4.a libdb4.a )
( cd usr/lib ; rm -rf libdb4.so )
( cd usr/lib ; ln -sf libdb-4.so libdb4.so )
